// MIT License
//
// Copyright (c) 2025 Cocotec Limited
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// SPDX-License-Identifier: MIT

#include "coco/stream_adapter.h"

// This is defined to 'inline' when this .cc file is #included in stream_adapter.h
#ifndef COCO_DETAIL_INLINED
#define COCO_DETAIL_INLINED
#endif

namespace coco {

inline namespace v1 {
namespace detail {

COCO_DETAIL_INLINED StreamBufferAdapter::StreamBufferAdapter(Formatter& formatter) : formatter_(formatter) {}

COCO_DETAIL_INLINED std::streamsize StreamBufferAdapter::xsputn(const char_type* s, std::streamsize size) {
  formatter_.write_data(s, static_cast<size_t>(size));
  return size;
}

COCO_DETAIL_INLINED int StreamBufferAdapter::overflow(int ch) {
  if (traits_type::not_eof(ch)) {
    formatter_.write_char(traits_type::to_char_type(ch));
  }
  return ch;
}

}  // namespace detail
}  // namespace v1
}  // namespace coco
